## Amenorrhea Data Example Code

## Packages
library(tidyverse)
library(mice)
library(knitr)
library(kableExtra)
library(geepack)
library(multcomp) # for glht()
library(wgeesel) # for IPW-GEE
library(gridExtra)
library(nlme) # lme() 
library(lme4) # glmer() 


## Color Palettes
pal2use <- c("#000000", "#E69F00", "#56B4E9", "#009E73", 
             "#F0E442", "#0072B2", "#D55E00", "#CC79A7")
pal2use2 <- c("#E69F00", "#0072B2")


## Load Data
# Clinical trial of contracepting women (Machin 1988, ALA p415)
ctcw <- read.csv("../Datasets/amenorrhea.csv", 
                 na.strings = ".") %>% 
  mutate(Dosage = ifelse(dose == 1, "150mg", "100mg"))
head(ctcw)


## Wide format
ctcw_wide <- ctcw %>% 
  pivot_wider(names_from = visit, values_from = amenorrhea)

# Missing data patterns (package mice) 
mdpatt <- md.pattern(ctcw_wide)


# prevalence by visit number and dosage
prev_by_time <- ctcw %>% 
  group_by(visit, Dosage) %>% 
  summarize(n = n(), prevalence = mean(amenorrhea, na.rm = TRUE))

ggplot(prev_by_time, aes(x=visit, y=prevalence, 
                        group=factor(Dosage), 
                        color=factor(Dosage))) + 
  geom_line() + 
  geom_point() + 
  theme_classic(base_size = 13) + 
  labs(color = "Dose") + 
  scale_color_manual(values = pal2use2) + 
  xlab("Time Point (quarterly visits)") + ylab("Prevalence of Amenorrhea") + 
  ggtitle("Prevalence of Amenorrhea by Dosage and Time")


## Create visit^2
ctcw$visit2 <- ctcw$visit^2


## Order data frame by id and visit
ctcw <- ctcw[order(ctcw$id, ctcw$visit),]


## Fit GEE models
m_ind <- geeglm(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
                id = id, data = ctcw, family = binomial, waves = visit,
                corstr = "independence")
m_exc <- geeglm(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
                id = id, data = ctcw, family = binomial, waves = visit,
                corstr = "exchangeable")
m_ar1 <- geeglm(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
                id = id, data = ctcw, family = binomial, waves = visit,
                corstr = "ar1")
m_uns <- geeglm(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
                id = id, data = ctcw, family = binomial, waves = visit,
                corstr = "unstructured")

m_glm <- glm(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
             data = ctcw, family = binomial)


## Results from working independence
summary(m_ind)


## Results from working exchangeable
summary(m_exc)


## Results from working AR1
summary(m_ar1)


## Results from working unstructured
summary(m_uns)


## Odds ratio at 4th visit (12 months)
pop_odds12 <- glht(m_exc, "4*visit:dose + 16*dose:visit2 = 0")
summary(pop_odds12)
confint(pop_odds12)


## Generalized linear mixed model with random intercepts
glmm.ri <- glmer(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose + (1 | id), 
                 data = ctcw, nAGQ = 10, family = "binomial")
summary(glmm.ri)


## Plot point estimates 
point_est_df <- data.frame(dose = rep(c(0,1), each = 4), 
                           Dosage = rep(c("100mg", "150mg"), each=4),  
                           visit = 1:4, visit2 = (1:4)^2)
point_est_df$glmm_est <- predict(glmm.ri, 
                                 newdata = point_est_df, 
                                 type = "response",  # probabilities 
                                 re.form = NA) # no random effects
point_est_df$glmm_logodds <- predict(glmm.ri, 
                                 newdata = point_est_df, 
                                 type = "link",  # log odds 
                                 re.form = NA) # no random effects
ctcw$pred_prob <- predict(glmm.ri, 
                          newdata = ctcw, 
                          type = "response",  # probabilities
                          re.form = NULL)  # all random effects 
ctcw$pred_logodds <- predict(glmm.ri, 
                          newdata = ctcw, 
                          type = "link",  # log odds
                          re.form = NULL)  # all random effects 

prob.plot <- ggplot(data = point_est_df, 
       aes(x=visit, y=glmm_est, group=Dosage, col=Dosage)) +
  geom_line(size = 1.5) + 
  geom_point(size = 2.5) +
  geom_line(data = ctcw, 
            aes(x=visit, y=pred_prob, group=id, col=Dosage), 
            size=0.25, linetype = 2) + 
  theme_classic(base_size = 16) + 
  ylab("Probability of Amenorrhea") + xlab("Visit Number")  + 
  theme(legend.position="bottom") + 
  scale_color_manual(values = pal2use2)
logodds.plot <- ggplot(data = point_est_df, 
       aes(x=visit, y=glmm_logodds, group=Dosage, col=Dosage)) +
  geom_line(size = 1.5) + 
  geom_point(size = 2.5) +
  geom_line(data = ctcw, 
            aes(x=visit, y=pred_logodds, group=id, col=Dosage), 
            size=0.25, linetype = 2) + 
  theme_classic(base_size = 16) + 
  ylab("Log Odds of Amenorrhea") + xlab("Visit Number")  + 
  theme(legend.position="bottom") + 
  scale_color_manual(values = pal2use2)
grid.arrange(logodds.plot, prob.plot, nrow=1)


## Estimated ratio at visit 4 
avg_odds12 <- glht(glmm.ri, "4*visit:dose + 16*dose:visit2 = 0")
summary(avg_odds12)
exp(0.504)   # exponentiate to get odds ratio 
confint(avg_odds12)
paste(round(exp(0.032), 3), 
      round(exp(0.976), 3), 
      sep=", ")  # exponentiate to get odds ratios 


## Wald test 
# Full model: visit, visit^2, and full interactions with dose 
glmm.ri.1 <- glmer(amenorrhea ~ visit + visit:dose + visit2 + 
                     visit2:dose + (1 | id), 
                 data = ctcw, nAGQ = 10, family = "binomial")
# Reduced model: visit, visit^2, no dose 
glmm.ri.0 <- glmer(amenorrhea ~ visit + visit2 + (1 | id), 
                 data = ctcw, nAGQ = 10, family = "binomial")
# Likelihood ratio test: 
anova(glmm.ri.1, glmm.ri.0) 


## Comparison with GEE using working exchangeable
m_exc <- geeglm(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
                id = id, data = ctcw, family = binomial, waves = visit,
                corstr = "exchangeable")
point_est_df$gee_est <- predict(m_exc, newdata = point_est_df, 
                                 type = "response")

point_est_df %>% 
  pivot_longer(c(glmm_est, gee_est), 
               names_to = "Model", 
               values_to = "Estimate") %>% 
  mutate(Model = ifelse(Model == "glmm_est", "GLMM (RI)", "GEE (Exch)")) %>% 
  ggplot(aes(x=visit, y=Estimate, group=interaction(Model, Dosage), 
             col=Dosage, linetype=Model)) +
  geom_line() + 
  geom_point()  +
  theme_classic(base_size = 16) + 
  ylab("Predicted P(Amenorrhea)") + xlab("Visit Number")  + 
  theme(legend.position="bottom") + 
  scale_color_manual(values = pal2use2) +
  guides(col=guide_legend(nrow=2,byrow=TRUE), 
         linetype=guide_legend(nrow=2, byrow=TRUE))


## Complete Case Analysis
# IDS where at least one visit is missing
ids_miss <- ctcw$id[is.na(ctcw$amenorrhea)]

# Create dataset for "complete case" analysis
ctcw_complete <- ctcw[!ctcw$id %in% ids_miss, ]

m_cc <- geeglm(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
               id = id, data = ctcw, family = binomial, 
               waves = visit, corstr = "exchangeable")

summary(m_cc)


## Available Data Analysis
ctcw_available <- ctcw[!is.na(ctcw$amenorrhea), ]
m_ad <- geeglm(amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
               id = id, data = ctcw_available, family = binomial, 
               waves = visit, corstr = "exchangeable")
summary(m_ad)


## -WGEE
library(wgeesel)
# Create indicator for dropout. R = 1 if observed; R = 0 if missing
ctcw$R <- 1 
ctcw$R[is.na(ctcw$amenorrhea)] <- 0
# Our dropout model includes the response at the previous time point as potentially being informative of dropout
ctcw$lag1y <- ylag(ctcw$id, ctcw$amenorrhea, 1)
head(ctcw)

m_ipw <- wgee(model = amenorrhea ~ visit + visit:dose + visit2 + visit2:dose,
              data = ctcw, id = ctcw$id,
              family="binomial", corstr ="exchangeable",
              scale = NULL, mismodel = R ~ dose + visit + lag1y*dose)

summary(m_ipw)

summary(m_ipw$mis_fit)


# Histogram of estimated weights where response is observed
hist(m_ipw$weight[ctcw$R==1], 
     xlab="Estimated IPW Weights", 
     main="Estimated Weights where Response is Observed")

