## Dental Growth Data Example Code

## Packages
library(tidyverse)
library(nlme)
library(geepack)
library(multcomp)     # glht() function for linear combinations of parameters
library(ggcorrplot)   # correlation heat maps 
library(GGally)       # scatterplot matrix 
library(cowplot)      # convenient for grids of plots 
library(clubSandwich) # robust SE for LMM

## Color palettes
pal2use <- c("#000000", "#E69F00", "#56B4E9", "#009E73", 
             "#F0E442", "#0072B2", "#D55E00", "#CC79A7")
pal2use2 <- c("#E69F00", "#0072B2")


## Load data
data("Orthodont") # in package nlme


# relevel Sex so that the reference group is female
Orthodont$Sex <- relevel(Orthodont$Sex, ref = "Female")


head(Orthodont)


## Wide format
dental.wide <- data.frame(Orthodont) %>% 
  pivot_wider(names_from = age, names_prefix = "Age", 
              values_from = distance)
head(dental.wide)


## Long format
dental.long <- dental.wide %>% 
  pivot_longer(Age8:Age14, 
               names_to = "age", names_prefix = "Age", 
               values_to = "distance") %>% 
  mutate(age = as.numeric(age)) %>% 
  dplyr::select(distance, age, Subject, Sex)

## make sure we got the same dataset back 
all(Orthodont == dental.long)

## here we will define a new variable age.8 = age - 8
dental.long$age.8 = dental.long$age - 8


any(is.na(dental.wide))


# Look at number of observations for each subject 
table(dental.long$Subject) 
# How many unique numbers of observations? 
length(unique(table(dental.long$Subject)))

## EDA plots
dental.long %>% 
  ggplot(aes(x=age, y=distance, group=Subject, col=Sex)) + 
  geom_line(aes(linetype=Sex)) + 
  geom_point() + 
  xlab("Age (Years)") + ylab("Length (mm)") + 
  ggtitle("Spaghetti Plot (Individuals' Dental Growth)") + 
  theme_classic(base_size = 14) + 
  scale_color_manual(values=pal2use2)

dental.long %>% 
  mutate(Subject = as.character(Subject)) %>% # more natural ordering 
  ggplot(aes(x=age, y=distance, col=Sex)) + 
  geom_line() + 
  geom_point() + 
  xlab("Age (Years)") + ylab("Length (mm)") + 
  theme_classic(base_size = 14) + 
  facet_wrap(vars(Subject)) + 
  scale_color_manual(values=pal2use2)


## Response profiles 
dental.summ <- dental.long %>% 
  group_by(Sex, age) %>% 
  summarise(Mean = mean(distance))

dental.summ %>% 
  ggplot(aes(x=age, y=Mean, group=Sex, col=Sex)) + 
  geom_line(aes(linetype=Sex)) + 
  geom_point() + 
  xlab("Age (Years)") + ylab("Average Length (mm)") + 
  ggtitle("Mean Response Profiles (Growth By Sex)")  + 
  theme_classic(base_size = 14) + 
  scale_color_manual(values=pal2use2)


## Scatterplot matrix (GGally package)
ggpairs(dental.wide[, c("Age8", "Age10", "Age12", "Age14")]) + 
  ggtitle("Dental Growth Correlation Structure")  + 
  theme_bw(base_size = 14) 


## Correlation heatmaps
cormat.boys <- dental.wide %>% 
  filter(Sex == "Male") %>% 
  dplyr::select(Age8:Age14) %>% 
  cor() %>% 
  round(., 3) 
cormat.boys 

cormat.girls <- dental.wide %>% 
  filter(Sex == "Female") %>% 
  dplyr::select(Age8:Age14) %>% 
  cor() %>% 
  round(., 3)
cormat.girls 

corplot.boys <- cormat.boys %>% 
  ggcorrplot(type = "lower") + 
  labs(title="Correlation Matrix for Boys") + 
  theme(legend.position = 'bottom')
corplot.girls <- cormat.girls %>% 
  ggcorrplot(type = "lower") + 
  labs(title="Correlation Matrix for Girls") + 
  theme(legend.position = 'bottom')

plot_grid(corplot.boys, corplot.girls, nrow=1)


## Order by subject id and age
dental.long <- dental.long[order(dental.long$Subject, dental.long$age),]

## GEE with working independence
mod1.gee <- geeglm(distance ~ age.8 * Sex, 
               id = Subject, data = dental.long)
summary(mod1.gee)

## Get predicted values for plot 
dental.long$fitted.linear <- mod1.gee$fitted[,1]

## Plot fitted and observed values 
ggplot(dental.long) + 
  geom_point(aes(x=age, y=distance)) + 
  geom_line(aes(x=age, y=fitted.linear), col = "blue") + 
  facet_wrap(vars(Sex), nrow=1)


tidy(mod1.gee, conf.int = T)


## Fit mean model using different working correlation structures
m_ind <- geeglm(distance ~ age.8*Sex, id = Subject,
                data = dental.long, corstr = "independence")
m_exc <- geeglm(distance ~ age.8*Sex, id = Subject,
                data = dental.long, corstr = "exchangeable")
m_ar1 <- geeglm(distance ~ age.8*Sex, id = Subject,
                data = dental.long, corstr = "ar1")
m_uns <- geeglm(distance ~ age.8*Sex, id = Subject,
                data = dental.long, corstr = "unstructured")

m_ols <- lm(distance ~ age.8*Sex, data=dental.long)


## Results from working exchangeable
summary(m_exc)


## Results from working AR1
summary(m_ar1)


## Results from working unstructured
summary(m_uns)

## Results (CIs) from working independence
tidy(m_ind, conf.int = T)


## Estimated slope for males and 95% CI
male_slope <- glht(m_ind, "age.8 + age.8:SexMale = 0")
summary(male_slope)
confint(male_slope)


## Fit age categorically rather than linearly
mod2.gee <- geeglm(distance ~ factor(age) * Sex, 
                   id = Subject, data = dental.long)
summary(mod2.gee)


## Get predicted values for plot 
dental.long$fitted.cat <- mod2.gee$fitted[,1]

## Plot fitted and observed values 
ggplot(dental.long) + 
  geom_point(aes(x=age, y=distance), shape = 1) + 
  geom_line(aes(x=age, y=fitted.linear), col = "blue") + 
  geom_point(aes(x=age, y=fitted.cat), col = "red", size = 2, shape=15) + 
  facet_wrap(vars(Sex), nrow=1)


## Linear Mixed Effects Models

## Linear mean model 
# random intercept 
mod1.ri <- lme(fixed = distance ~ age.8 * Sex, 
               random = ~ 1 | Subject, 
               data = dental.long)
summary(mod1.ri)
intervals(mod1.ri)


# Level 0 fitted values 
fixed.pred <- fitted(mod1.ri, level=0)
head(fixed.pred)
head(matrix(fixed.pred, ncol=4, byrow=T))


# Level 1 fitted values 
randint.pred <- fitted(mod1.ri, level=1)
head(randint.pred)

head(matrix(randint.pred, ncol=4, byrow=T))


dental.long$fitted.lme <- fitted(mod1.ri)

# All subject-specific trajectories 
dental.long %>% 
  ggplot() + 
  geom_line(aes(x=age, y=distance, col=Sex)) + 
  geom_point(aes(x=age, y=distance, col=Sex)) + 
  geom_line(aes(x=age, y=fitted.lme), col="black", lty = 2, size = 1) + 
  xlab("Age (Years)") + ylab("Distance (mm)") + 
  theme_classic(base_size = 14) + 
  facet_wrap(vars(Subject), nrow=4) + 
  scale_color_manual(values = pal2use2) + 
  theme(legend.position = "right")

# Zooming in on two subjects 
dental.long %>% 
  filter(Subject %in% c("M10", "M11")) %>% 
  ggplot() + 
  geom_line(aes(x=age, y=distance, col=Subject)) + 
  geom_point(aes(x=age, y=distance, col=Subject)) + 
  geom_line(aes(x=age, y=fitted.lme, col=Subject), lty = 2) + 
  xlab("Age (Years)") + ylab("Distance (mm)") + 
  ggtitle("Comparison Between Two Male Subjects") + 
  theme_classic(base_size = 14) + 
  theme(legend.position = "right") + 
  scale_color_manual(values = pal2use2)


## random intercept, random slope 
mod1.rs <- lme(fixed = distance ~ age.8 * Sex, 
               random = ~ 1 + age | Subject, 
               data = dental.long)
summary(mod1.rs)

# Level 0 fitted values 
fixed.pred.2 <- fitted(mod1.rs, level=0)
head(matrix(fixed.pred.2, ncol=4, byrow=T))


# Level 1 fitted values 
randsl.pred <- fitted(mod1.rs, level=1)
head(randsl.pred)
head(matrix(randsl.pred, ncol=4, byrow=T))


dental.long$fitted.lme.rs <- fitted(mod1.rs)
dental.long %>% 
  ggplot() + 
  geom_line(aes(x=age, y=distance, col=Sex)) + 
  geom_point(aes(x=age, y=distance, col=Sex)) + 
  geom_line(aes(x=age, y=fitted.lme.rs), col="black", lty = 2) + 
  xlab("Age (Years)") + ylab("Distance (mm)") + 
  theme_classic(base_size = 14) + 
  facet_wrap(vars(Subject), nrow=4) + 
  theme(legend.position = "right") + 
  scale_color_manual(values=pal2use2)


coef(mod1.rs)[c("M10", "M11"), ]

dental.long %>% 
  filter(Subject %in% c("M10", "M11")) %>% 
  ggplot() + 
  geom_line(aes(x=age, y=distance, col=Subject)) + 
  geom_point(aes(x=age, y=distance, col=Subject)) + 
  geom_line(aes(x=age, y=fitted.lme, col=Subject), lty = 3) + 
  geom_line(aes(x=age, y=fitted.lme.rs, group=Subject), col="black", size=1.25, lty = 2) + 
  xlab("Age (Years)") + ylab("Distance (mm)") + 
  ggtitle("Comparison Between Two Male Subjects") + 
  theme_classic(base_size = 14) + 
  theme(legend.position = "right")


# Is the correlation structure better with random slopes? 
# Doesn't look like it (p=0.56, AIC and BIC are slightly lower for RI)
anova(mod1.rs, mod1.ri)


## Original CIs 
orig.tab <- cbind(summary(mod1.ri)$tTable, intervals(mod1.ri)$fixed)
round(orig.tab[, c("Value", "p-value", "lower", "upper")], 3)


# clubSandwich 
## Robust standard errors (random intercept model)
sig <- vcovCR(mod1.ri, type = "CR2")
summ <- coef_test(mod1.ri, sig)
cis <- conf_int(mod1.ri, sig)
robtab <- merge(summ[, c("Coef", "beta", "p_Satt")], 
      cis[, c("Coef", "CI_L", "CI_U")])
rownames(robtab) <- robtab$Coef 
robtab$Coef <- NULL 
round(robtab[c(1,2,4,3), ], 3)


# use maximum likelihood estimation to compare fixed effects 
# linear model 
modLIN.ri <- lme(fixed = distance ~ age * Sex, 
                 method = "ML",
                 random = ~ 1 | Subject,
                 data = dental.long)

# response profile model 
# note we add only two of the four ages because any two points can be fit with a line
modCAT.ri <- lme(fixed = distance ~ age * Sex + I(age == 10) * Sex + I(age == 12) * Sex, 
                 method = "ML",
                 random = ~ 1 | Subject,
                 data = dental.long)

# LRT 
anova(modLIN.ri, modCAT.ri)

