## Six Cities Data Example Code

## Packages
library(tidyverse)
library(lattice)
library(geepack)


## Read in Data
dat <- read.csv("../Datasets/Topeka.csv", row.names=1)
head(dat)


# age 0 is the age of entry
dat <- dat %>% 
  mutate(time_from_entry = age - age0)

## Some EDA plots
dat %>% 
  filter(age==age0) %>%
  ggplot(aes(x=age0)) +
  geom_histogram(fill="white", color="black", binwidth = 0.5) +
  labs(title="",
       x="Age at entry",
       y = "Count")+
  theme_classic(base_size = 15)

dat %>% 
  ggplot(aes(x=age, y=logFEV1)) +
  geom_point()+
  labs(title="",
       x="Age",
       y = "Log(FEV1)")+
  theme_classic(base_size = 15)

dat %>% 
  ggplot(aes(x=time_from_entry, y=logFEV1)) +
  geom_point()+
  labs(title="",
       x="Time from entry",
       y = "Log(FEV1)")+
  theme_classic(base_size = 15)

## Step function

# Create indicators for each age category 
dat$age_cat <- cut(dat$age, c(0, 8, 10, 12, 14, 16, Inf))

# Fit the step function model 
mod1 <- geeglm(logFEV1 ~ age_cat, id = id, data = dat)
summary(mod1)

# Add fitted values to dataset and then plot (with nice colors) 
dat$fitted1 <- mod1$fitted

pal2use <- c("#E69F00", "#0072B2")
xyplot(logFEV1 ~ age , data=dat, 
       panel=function(x,y){
         panel.xyplot(x, y, pch=16, col=pal2use[2])
         panel.xyplot(dat$age, dat$fitted1, pch=16, col=pal2use[1])
       })


## Linear

mod2 <- geeglm(logFEV1 ~ age, id = id, data = dat)
summary(mod2)

dat$fitted2 <- mod2$fitted
xyplot(logFEV1 ~ age , data=dat, 
       panel=function(x,y){
         panel.xyplot(x, y, pch=16, col=pal2use[2])
         panel.xyplot(dat$age, dat$fitted2, pch=16, col=pal2use[1])
       })


## Polynomial (Quadratic)
# I() notation allows you to include mathematical operators in a formula 
# Alternatively, you could create an age.2 = age^2 variable in the dataset 
mod3 <- geeglm(logFEV1 ~ age + I(age^2), id = id, data = dat)
summary(mod3)

dat$fitted3 <- mod3$fitted
xyplot(logFEV1 ~ age , data=dat, 
       panel=function(x,y){
         panel.xyplot(x, y, pch=16, col=pal2use[2])
         panel.xyplot(dat$age, dat$fitted3, pch=16, col=pal2use[1])
       })


## Linear Splines

# Create variables for "years since age X" (0 if age < age X)
dat$ageSpline10 <- pmax(dat$age - 10, 0)
dat$ageSpline14 <- pmax(dat$age - 14, 0)

# Model fitting and results 
mod4 <- geeglm(logFEV1 ~ age + ageSpline10 + ageSpline14, 
               id = id, data = dat)
summary(mod4)

# Add fitted values to dataset and plot 
dat$fitted4 <- mod4$fitted
xyplot(logFEV1 ~ age , data=dat, 
       panel=function(x,y){
         panel.xyplot(x, y, pch=16, col=pal2use[2])
         panel.xyplot(dat$age, dat$fitted4, pch=16, col=pal2use[1])
       })

